/*
 * Headerfile of dfu_func.c
 *
 * dfu_func provides the data structures, complex down- / upload functions
 * and dfu-suffix checking / handling functions
 *
 * Author: Jan Alte, DO1FJN
 *
 *
 * This file is part of the DFU Serial Loader (dfu-loader).
 * For general infomation about dfu-loader see "dfu_serial_main.c".
 *
 * dfu-loader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * dfu-loader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __DFU_FUNC_H__
#define __DFU_FUNC_H__

//! DFU-specific requests

typedef enum {
  DFU_DETACH=0, DFU_DNLOAD, DFU_UPLOAD, DFU_GETSTATUS,
  DFU_CLRSTATUS, DFU_GETSTATE, DFU_ABORT
} tDFUrequests;


typedef enum {
  DFUs_GETDEVICEDESCR=0x40, DFUs_GETFUNCTIONDESCR, DFUs_GETSTRING
} tDFUserialrequests;


typedef enum {
  staOK=0, errTARGET, errFILE, errWRITE,
  errERASE, errCHECK_ERASED, errPROG, errVERIFY,
  errADDRESS, errNOTDONE, errFIRMWARE, errVENDOR,
  errUSBR, errUNKNOWN, errSTALLEDPKT
} tDFUStatus;


typedef enum {
  appIDLE=0, appDETACH, dfuIDLE, dfuDNLOAD_SYNC,
  dfuDNLOAD_BUSY, dfuDNLOAD_IDLE, dfuMANIFEST_SYNC, dfuMANIFEST,
  dfuMANIFEST_WAIT_RESET, dfuUPLOAD_IDLE, dfuERROR
} tDFUDeviceState;


typedef struct __attribute__((__packed__)) {
  unsigned short bcdDevice;
  unsigned short idProduct;	
  unsigned short idVendor;	//
  unsigned short bcdDFU;	//
  char		 DFUsignature[3];
  unsigned char  length;
  unsigned long  crc;		// 32bit CR
} tDFUfilesuffix;


typedef struct {
  tDFUStatus		Status;
  tDFUDeviceState	State;
  unsigned int		PollTimeout;
  unsigned char		Index;
} tDFUStatusInfo;


typedef struct __attribute__((__packed__)) {
  unsigned char	 Length;	// Size of this descriptor in bytes
  unsigned char	 Type;		// DEVICE descriptor type
  unsigned short bscUSB;	// Binay Coded Decimal Spec. release
  unsigned char	 DeviceClass;	// Class code assigned by the USB
  unsigned char	 DeviceSubClass;// Subclass code assigned by the USB
  unsigned char	 DeviceProtocol;// Protocol code assigned by the USB
  unsigned char	 MaxPacketSize0;// Max packet size for EP0
  unsigned short idVendor;	// Vendor ID
  unsigned short idProduct;	// Product ID assigned by the manufacturer
  unsigned short bcdDevice;	// Device release number
  unsigned char	 iManufacturer;	// Index of manu. string descriptor
  unsigned char	 iProduct;	// Index of prod. string descriptor
  unsigned char	 iSerialNr;	// Index of S.N.  string descriptor
  unsigned char	 NumConfig;	// Number of possible configurations
} tDFUdevicedescriptor;


typedef struct __attribute__((__packed__)) {
  unsigned char	 Length;	// Size of this descriptor in bytes
  unsigned char  Type;		// DFU FUNCTIONAL descriptor type
  unsigned char  Attrib;	// DFU attributes
  unsigned short DetachTimeOut;	// Time, in milliseconds, that the device will wait after receipt of the DFU_DETACH request
  unsigned short TransferSize;	// Maximal number of bytes that the device can accept per control-write transaction
  unsigned short DFUVersion;	// BCD numeric expression identifying the version of the DFU Specification release
} tDFUfunctionaldescriptor;


// Funktionen
const char *dfu_status_str(tDFUStatus Status);
const char *dfu_state_str(tDFUDeviceState State);

void print_DFUinfo(const tDFUStatusInfo *Info);	// print "DFU_STATE: Status" to stdout

int	dfu_append_filesuffix(int FILEfd, tDFUfilesuffix *Suffix);
int	dfu_read_filesuffix(int FILEfd, tDFUfilesuffix *Suffix, unsigned long *fwlength);
void	dfu_show_suffixinfo(const tDFUfilesuffix *Suffix, int suffixlen, unsigned long fwlength);

int	dfu_test_filesuffix(const char *filename);

/*
 dfu_append_filesuffix() fills bcdDFU, Length and Signature with the correct values then
 append Suffix to the open file in reverse order.
 Return 0 if success otherwise -1
*/
int	dfu_append_filesuffix(int FILEfd, tDFUfilesuffix *Suffix);


int	dfu_serial_verify(int SERfd, const char *filename);
int	dfu_serial_upload(int SERfd, const char *filename);
int	dfu_serial_download(int SERfd, const char *filename);


#endif