/*
 * DFU Serial Loader Main File
 * ---------------------------
 * This project is firmware of the AVR32/C5 DV-Modem, developed by J.Alte.
 * It processes various converting modes for digitized speech signals.
 * See www.digisolutions.de (german only) for more infomation about DV-Modem.
 *
 * Copyright (c) 2010 by Jan Alte, DO1FJN
 *
 *
 * dfu_serial_main.c (main source file of dfu-loader)
 *
 * Author: Jan Alte, DO1FJN
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#define PRG_NAME	"dfu-loader"
#define VERSIONSTR	"Version 0.1\n"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include <errno.h>

#include "checksums.h"
#include "fileIO.h"
#include "serialIO.h"
#include "dfu_func.h"

#define false	0
#define true	1


void show_help(void) {
  printf("Usage: "PRG_NAME" [options] ...\n"
         "  -h --help\t\t\tPrint this help message\n"
         "  -V --version\t\t\tPrint the version number\n"
         "  -d --device /dev/ttySx\tSelect serial mode on specified port\n"
         "  -I --info file\t\tshows dfu filesuffix from <file>\n"
         "  -D --download file\t\tWrite firmware from <file> into device\n"
         "  -U --upload file\t\tRead firmware from device into <file>\n"
	 "  -C --compare file\t\tCompare loaded firmware with <file> \n\n");
}


const static char prg_options[] =
  "hVI:D:U:C:d:";

const static struct option prg_optslong[] = {
  { "help", 0, 0, 'h' },
  { "version", 0, 0, 'V' },
  { "info", 1, 0, 'I' },
  { "download", 1, 0, 'D' },
  { "upload", 1, 0, 'U' },
  { "compare", 1, 0, 'C' },
  { "device", 1, 0, 'd' }
};


typedef enum {
  lmINIT=0, lmTESTFILE, lmDOWNLOAD, lmUPLOAD, lmVERIFY, lmERROROPTIONS
} tDFUloaderMode;

#define NEED_CONNECTION(Mode)	((Mode==lmDOWNLOAD)||(Mode==lmUPLOAD)||(Mode==lmVERIFY))


int main(int argc, char **argv) {
  int exitcode=0, option;
  char *devicename = NULL;
  char *filename = NULL;
  int SERfd;
  
  tDFUloaderMode Mode = lmINIT;

  printf("\nDFU serial loader - (C) 2010 by DO1FJN / digisolutions.de\n"
         "This program is Free Software and has ABSOLUTELY NO WARRANTY\n\n");
  do {
    int option_index = 0;
    option = getopt_long(argc, argv, prg_options, prg_optslong, &option_index);
    switch (option) {
      case 'h':
        show_help();
        exit(2);
        //option = -1;
        break;
      case 'V':
	printf(VERSIONSTR);
	break;
      case 'd':
	devicename = optarg;
        break;
      case 'I':
        if (Mode==lmINIT) {
	  Mode = lmTESTFILE;
	  filename = optarg;
	} else
	  Mode = lmERROROPTIONS;
        break;
      case 'D':
        if (Mode==lmINIT) {
	  Mode = lmDOWNLOAD;
	  filename = optarg;
	} else
	  Mode = lmERROROPTIONS;
        break;
      case 'U':
        if (Mode==lmINIT) {
	  Mode = lmUPLOAD;
	  filename = optarg;
	} else
	  Mode = lmERROROPTIONS;
        break;
      case 'C':
        if (Mode==lmINIT) {
	  Mode = lmVERIFY;
	  filename = optarg;
	} else
	  Mode = lmERROROPTIONS;
        break;
    } // hctiws
  } while (option != -1);
  // *** DO THE JOB NOW ***
  SERfd = 0;			// init to "not used"
  if (NEED_CONNECTION(Mode)) {	// need a connection to Device
     SERfd = SER_open(devicename);
  } // fi CON needed
  if ((Mode > lmINIT) && (filename==NULL)) {
    fprintf(stderr, "You need to specify a filename to -D -U -C or -T\n");
  } else switch(Mode) {
    case lmINIT:	// Do nothing
      break;
    case lmTESTFILE:
      exitcode = dfu_test_filesuffix(filename);
      break;
    case lmDOWNLOAD:
      exitcode = dfu_serial_download(SERfd, filename);
      break;
    case lmUPLOAD:
      exitcode = dfu_serial_upload(SERfd, filename);
      break;
    case lmVERIFY:
      exitcode = dfu_serial_verify(SERfd, filename);
      break;
    default:
      fprintf(stderr, "Too many options!\n");
      show_help();
      exitcode = -2;
      break;
  } // hctiws
  if (SERfd) {
    usleep(125000);
    SER_close(SERfd);
  }
  printf("\nDONE.\n");
  return exitcode;
}
