/*
 * Headerfile of serialIO.c
 *
 * simple device file based serial functions
 *
 * Author: Jan Alte, DO1FJN
 *
 *
 * This file is part of the DFU Serial Loader (dfu-loader).
 * For general infomation about dfu-loader see "dfu_serial_main.c".
 *
 * dfu-loader is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * dfu-loader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _SERIALIO_H
#define _SETIALIO_H

#define ACK		6
#define NAK		15


#define SER_DEVICE	"/dev/ttyS0"
#define SER_BUFSIZE	(8192+8)
#define SER_Baudrate	B115200

#define SER_Timeout	20		// RX-Timeout in nx10ms

#define SER_PollDelay	125000		// 125ms
#define SER_RETRIES	3

#define SER_TXTIMEOUT			60		// min. 60 Bits idle after Sending
#define SER_TXWAITTIME			(1000000L*(SER_TXTIMEOUT+1)/115200L)
// SER_TXWAITTIME: Zeit in µs, die das Gerät benötigt um Transferende zu erkennen
#define SER_CALCTRANSFERDELAY(sbytes)	((sbytes*1000000L / 11520L)+SER_TXWAITTIME)	// Zeit in µs


#define FRAMESTARTID	0xD0		// RS232-Paket-Startzeichen

typedef struct __attribute__((__packed__)) {
  unsigned char		id;
  unsigned short	len;
  unsigned char		cmd;
} tRS232pktHeader;


typedef struct __attribute__((__packed__)) {
  union {
    tRS232pktHeader h;
    char data[4];
  };
} tRS232header;


typedef struct __attribute__((__packed__)) {
  union {
    tRS232pktHeader h;
    char data[SER_BUFSIZE];
  };
} tRS232buffer;


int SER_open(const char *devicefile);  // Öffnen, Rückgabe: FileDecriptor für IO
int SER_close(int SERfd); // Schliessen der Schnittstelle (SERfd)

int SER_SendBlock(int SERfd, unsigned char cmd, const char *buffer, unsigned short size);
int SER_Receive(int SERfd, char *rxbuf, unsigned int maxsize);

/*
  SER_SendReceive() sendet Paket Cmd,SendBuf bis zu SER_RETRIES mal aus, solange bis eine Antwort
  eintrifft.
  Rückgabe -1 wenn keine Antwort, -EINVAL wenn SendLen oder MaxReceive zu groß angegeben oder
  Anzahl der empfangenen Bytes, falls eine Antwort (in ReceiveBuf) eingetroffen ist.
  Für die Antwort hat das Gerät SER_PollDelay plus Übertragungszeit (SendLen+MaxReceive) Zeit.
  Daher sollte MaxReceive ca. der Größe des erwarteten Paketes entsprechen.
*/
int SER_SendReceive(int SERfd, char Cmd, const char *SendBuf, unsigned short SendLen, char *ReceiveBuf, unsigned short MaxReceive);


#endif
